{**********************************************************}
{* 4-bit-full-subtractor unit for LogicGates4 application *}
{**********************************************************}

unit subtractor4f;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, FileUtil, Forms, Controls, Graphics, Dialogs, ExtCtrls, StdCtrls, Grids, common;

type
  {****************}
  { TfSubtractor4f }
  {****************}
  TfSubtractor4f = class(TForm)
    Shape1: TShape;
    StaticText1: TStaticText;
    Label1, Label2, Label8, Label10, Label18, Label9: TLabel;
    laA1, laA2, laA4, laA8, laB1, laB2: TLabel;
    laB4, laB8, laS1, laS2, laS4, laS8: TLabel;
    Shape20, Shape21, Shape22, Shape23, Shape25, Shape28, Shape29, Shape32: TShape;
    Shape35, Shape36, Shape37, Shape38, Shape39, Shape45, Shape50, Shape51: TShape;
    Shape52, Shape53, Shape54, Shape55, Shape57, Shape58, Shape60, Shape61: TShape;
    Shape62, Shape63, Shape64, Shape65, Shape66, Shape67, Shape68, Shape69: TShape;
    Shape70, Shape71, Shape72, Shape73, Shape74, Shape75, Shape76, Shape77: TShape;
    Shape78, Shape79, Shape80, Shape81, Shape82, Shape83, Shape84, Shape85: TShape;
    Shape86, Shape87, Shape88, Shape89, Shape90, Shape91, Shape92, Shape93: TShape;
    Shape94, Shape95, Shape96, Shape97, Shape98, Shape99, Shape100, Shape101: TShape;
    Shape102, Shape103, Shape104, Shape105, Shape106, Shape107, Shape108, Shape109: TShape;
    Shape110, Shape111, Shape112, Shape113, Shape114, Shape115, Shape116, Shape117: TShape;
    Shape118, Shape119, Shape120, Shape121, Shape122, Shape123: TShape;
    Shape124, Shape125, Shape126, Shape127, Shape128: TShape;
    Image1, Image2, Image3, Image4, Image5, Image6, Image7, Image8, Image9: TImage;
    imSwitchA1, imSwitchA2, imSwitchA4, imSwitchA8: TImage;
    imSwitchB1, imSwitchB2, imSwitchB4, imSwitchB8: TImage;
    shSwitchA1a, shSwitchA1b, shSwitchA2a, shSwitchA2b: TShape;
    shSwitchA4a, shSwitchA4b, shSwitchA8a, shSwitchA8b: TShape;
    shSwitchB1a, shSwitchB1b, shSwitchB2a, shSwitchB2b: TShape;
    shSwitchB4a, shSwitchB4b, shSwitchB8a, shSwitchB8b: TShape;
    shLedA1, shLedA2, shLedA4, shLedA8: TShape;
    shLedB1, shLedB2, shLedB4, shLedB8: TShape;
    shLedS1, shLedS2, shLedS4, shLedS8, shLedCout: TShape;
    sgSub: TStringGrid;
    btClose: TButton;
    procedure FormCreate(Sender: TObject);
    procedure FormActivate(Sender: TObject);
    procedure imSwitchA1Click(Sender: TObject);
    procedure imSwitchA2Click(Sender: TObject);
    procedure imSwitchA4Click(Sender: TObject);
    procedure imSwitchA8Click(Sender: TObject);
    procedure imSwitchB1Click(Sender: TObject);
    procedure imSwitchB2Click(Sender: TObject);
    procedure imSwitchB4Click(Sender: TObject);
    procedure imSwitchB8Click(Sender: TObject);
    procedure btCloseClick(Sender: TObject);
  private
    aLeds: TLeds;
    aSwitches: TSwitches;
    bSwitches, bLeds: TStatus;
  end;

const
  // Switches description: Data = objects positions (transparent image, long bar shape, short bar shape), horiz. closing move, vert. closing move
  DSwitches: array[0..7] of TSwitchData = (
    (Name: 'SwitchA1';
     Data: (512, 534, 514, 0, 1)),
    (Name: 'SwitchA2';
     Data: (378, 400, 380, 0, 1)),
    (Name: 'SwitchA4';
     Data: (246, 268, 248, 0, 1)),
    (Name: 'SwitchA8';
     Data: (112, 134, 114, 0, 1)),
    (Name: 'SwitchB1';
     Data: (577, 599, 579, 0, 1)),
    (Name: 'SwitchB2';
     Data: (444, 466, 446, 0, 1)),
    (Name: 'SwitchB4';
     Data: (310, 333, 313, 0, 1)),
    (Name: 'SwitchB8';
     Data: (177, 200, 180, 0, 1))
  );

var
  fSubtractor4f: TfSubtractor4f;

implementation

{$R *.lfm}

{ 4-bit-full-subtractor: Set switch position; do binary subtraction; turn LEDs on/off; fill-in subtraction-table }

procedure SwitchAndAdd(DSwitches: array of TSwitchData; Switches: TSwitches; var SwitchStatus: TStatus; Leds: TLeds; var LedStatus: TStatus; IX: Integer);

var
  Exp, I: Integer;
  A, B, C, Sout, Cout, S1, S2, C1, C2: Boolean;
  Dec: array[0..3] of Integer;
  Bin, Compl: TBinary;

begin
  // Change current input from 0 to Vs (and vice-versa)
  SwitchStatus[IX] := not SwitchStatus[IX];
  // Corresponding LED status
  LedStatus[IX] := SwitchStatus[IX];
  // Set switch positions
  SwitchesOnOff(DSwitches, Switches, SwitchStatus);
  // Do the subtraction, using 4 1-bit-full-adders, adding A and the 2's complement of B, as is done in a real circuit
  Cout := True;                                                                // Cin of first adder set to Vs (in order to form 2's complement)
  for I := 0 to 3 do begin
    A := LedStatus[I];                                                         // actual bit for A input
    B := not LedStatus[I + 4];                                                 // actual bit for B input (inverted in order to form 1's complement)
    C := Cout;                                                                 // Cin = previous Cout
    // 1-bit full-adder binary addition
    Adder(A, B, C, S1, C1, S2, C2, Sout, Cout);
    // Actual bit sum LED status
    LedStatus[I + 8] := SOut;
  end;
  // For a binary subtraction, Cout of the last adder may be used to indicate the polarity of the subtraction result
  LedStatus[12] := Cout;
  // Turn all LEDs on/off
  LedsOnOff(Leds, LedStatus);
  // Fill binary values into stringgrid
  for I := 0 to 3 do begin
    fSubtractor4f.sgSub.Cells[4 - I, 1] := ' ' + IntToStr(Bits[LedStatus[I]]);
    fSubtractor4f.sgSub.Cells[4 - I, 2] := ' ' + IntToStr(Bits[LedStatus[I + 4]]);
    fSubtractor4f.sgSub.Cells[4 - I, 3] := ' ' + IntToStr(Bits[LedStatus[I + 8]]);
    Bin[3 - I] := LedStatus[I + 8];
  end;
  // Calculate and fill in result's 2nd complement
  Compl := Complement2(Bin);
  for I := 0 to 3 do
    fSubtractor4f.sgSub.Cells[I + 1, 4] := ' ' + IntToStr(Bits[Compl[I]]);
  // Calculate decimal values
  Dec[0] := 0; Dec[1] := 0; Dec[2] := 0; Exp := 1;
  for I := 0 to 3 do begin
    Dec[0] += Bits[LedStatus[I]] * Exp; Dec[1] += Bits[LedStatus[I + 4]] * Exp; Dec[2] += Bits[LedStatus[I + 8]] * Exp;
    Exp *= 2;
  end;
  // Decimal result may be positive or negative
  if Dec[0] < Dec[1] then
    Dec[2] := -(16 - Dec[2]);
  // Fill-in decimal values
  for I := 0 to 2 do begin
    fSubtractor4f.sgSub.Cells[5, I + 1] := ' ' + IntToStr(Dec[I]);
    if Abs(Dec[I]) < 10 then
      fSubtractor4f.sgSub.Cells[5, I + 1] := ' ' + fSubtractor4f.sgSub.Cells[5, I + 1];
    if Dec[I] >= 0 then
      fSubtractor4f.sgSub.Cells[5, I + 1] := ' ' + fSubtractor4f.sgSub.Cells[5, I + 1];
  end;
end;

{****************}
{ TfSubtractor4f }
{****************}

{ Application start: Initialisation }

procedure TfSubtractor4f.FormCreate(Sender: TObject);

begin
  // Arrays with switches (transparent image and bar shapes) and their "on/off" status
  SetLength(aSwitches, 8); SetLength(bSwitches, 8);
  aSwitches[0].Switch0 := imSwitchA1; aSwitches[0].Switch1 := shSwitchA1a; aSwitches[0].Switch2 := shSwitchA1b;
  aSwitches[1].Switch0 := imSwitchA2; aSwitches[1].Switch1 := shSwitchA2a; aSwitches[1].Switch2 := shSwitchA2b;
  aSwitches[2].Switch0 := imSwitchA4; aSwitches[2].Switch1 := shSwitchA4a; aSwitches[2].Switch2 := shSwitchA4b;
  aSwitches[3].Switch0 := imSwitchA8; aSwitches[3].Switch1 := shSwitchA8a; aSwitches[3].Switch2 := shSwitchA8b;
  aSwitches[4].Switch0 := imSwitchB1; aSwitches[4].Switch1 := shSwitchB1a; aSwitches[4].Switch2 := shSwitchB1b;
  aSwitches[5].Switch0 := imSwitchB2; aSwitches[5].Switch1 := shSwitchB2a; aSwitches[5].Switch2 := shSwitchB2b;
  aSwitches[6].Switch0 := imSwitchB4; aSwitches[6].Switch1 := shSwitchB4a; aSwitches[6].Switch2 := shSwitchB4b;
  aSwitches[7].Switch0 := imSwitchB8; aSwitches[7].Switch1 := shSwitchB8a; aSwitches[7].Switch2 := shSwitchB8b;
  // Arrays with LEDs (shape) and their "on/off" status
  SetLength(aLeds, 13); SetLength(bLeds, 13);
  aLeds[0] := shLedA1; aLeds[1] := shLedA2; aLeds[2] := shLedA4; aLeds[3] := shLedA8;
  aLeds[4] := shLedB1; aLeds[5] := shLedB2; aLeds[6] := shLedB4; aLeds[7] := shLedB8;
  aLeds[8] := shLedS1; aLeds[9] := shLedS2; aLeds[10] := shLedS4; aLeds[11] := shLedS8; aLeds[12] := shLedCout;
  // Use subscripts in input/output labels
  laA1.Caption := 'A' + SUB_Digits[0]; laA2.Caption := 'A' + SUB_Digits[1];
  laA4.Caption := 'A' + SUB_Digits[2]; laA8.Caption := 'A' + SUB_Digits[3];
  laB1.Caption := 'B' + SUB_Digits[0]; laB2.Caption := 'B' + SUB_Digits[1];
  laB4.Caption := 'B' + SUB_Digits[2]; laB8.Caption := 'B' + SUB_Digits[3];
  laS1.Caption := 'S' + SUB_Digits[0]; laS2.Caption := 'S' + SUB_Digits[1];
  laS4.Caption := 'S' + SUB_Digits[2]; laS8.Caption := 'S' + SUB_Digits[3];
end;

{ Form show-up: Initialize 4-bit full-subtractor simulation }

procedure TfSubtractor4f.FormActivate(Sender: TObject);

var
  I: Integer;

begin
  // All inputs at 0
  for I := 0 to Length(bSwitches) - 1 do
    bSwitches[I] := False;
  SwitchesOnOff(DSwitches, aSwitches, bSwitches);
  // All LEDs off
  for I := 0 to Length(bLeds) - 1 do
    bLeds[I] := False;
  LedsOnOff(aLeds, bLeds);
end;

{ Switch "A0, A1..." / "B0, B1..." activated: Update 4-bit-full-subtractor simulation for actual switch positions }

procedure TfSubtractor4f.imSwitchA1Click(Sender: TObject);

begin
  SwitchAndAdd(DSwitches, aSwitches, bSwitches, aLeds, bLeds, 0);
end;

procedure TfSubtractor4f.imSwitchA2Click(Sender: TObject);

begin
  SwitchAndAdd(DSwitches, aSwitches, bSwitches, aLeds, bLeds, 1);
end;

procedure TfSubtractor4f.imSwitchA4Click(Sender: TObject);

begin
  SwitchAndAdd(DSwitches, aSwitches, bSwitches, aLeds, bLeds, 2);
end;

procedure TfSubtractor4f.imSwitchA8Click(Sender: TObject);

begin
  SwitchAndAdd(DSwitches, aSwitches, bSwitches, aLeds, bLeds, 3);
end;

procedure TfSubtractor4f.imSwitchB1Click(Sender: TObject);

begin
  SwitchAndAdd(DSwitches, aSwitches, bSwitches, aLeds, bLeds, 4);
end;

procedure TfSubtractor4f.imSwitchB2Click(Sender: TObject);

begin
  SwitchAndAdd(DSwitches, aSwitches, bSwitches, aLeds, bLeds, 5);
end;

procedure TfSubtractor4f.imSwitchB4Click(Sender: TObject);

begin
  SwitchAndAdd(DSwitches, aSwitches, bSwitches, aLeds, bLeds, 6);
end;

procedure TfSubtractor4f.imSwitchB8Click(Sender: TObject);

begin
  SwitchAndAdd(DSwitches, aSwitches, bSwitches, aLeds, bLeds, 7);
end;

{ Button "Close": Close the window }

procedure TfSubtractor4f.btCloseClick(Sender: TObject);

begin
  Close;
end;

end.

