{*****************************************}
{* Main unit for LogicGates4 application *}
{*****************************************}

// Implementation of the interactive graphics:
//   - the circuit layout is constituted by a mixture of shapes and images
//   - the LEDs are colored ellipses; different color styles are used for on/off status
//   - the movement of the switches is done by changing the horizontal or vertical position (LEFT or TOP property) of the corresponding shapes
//   - the switch shapes being very small and difficult to click by the user, I used a transperant image, placed over them;
//     the OnClick event of the image is used to trigger the actions to be done when the corresponding switch has been clicked.

unit subtractors;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, FileUtil, Forms, Controls, Graphics, Dialogs, Menus, StdCtrls, ExtCtrls,
  subtractor1h, subtractor1f, subtractor4f, addsub4f, help, common;

type
  {***************}
  { TfSubtractors }
  {***************}
  TfSubtractors = class(TForm)
    mMenu: TMainMenu;
    mFile, mFileExit: TMenuItem;
    mHelp, mHelpHelp, mHelpAbout: TMenuItem;
    StaticText1: TStaticText;
    Memo1, Memo2, Memo3, Memo4: TMemo;
    btSubtractor1h: TButton;
    btSubtractor1f: TButton;
    btSubtractor4f: TButton;
    btAddSub4f: TButton;
    procedure FormCreate(Sender: TObject);
    procedure mFileExitClick(Sender: TObject);
    procedure mHelpHelpClick(Sender: TObject);
    procedure mHelpAboutClick(Sender: TObject);
    procedure btSubtractor1hClick(Sender: TObject);
    procedure btSubtractor1fClick(Sender: TObject);
    procedure btSubtractor4fClick(Sender: TObject);
    procedure btAddSub4fClick(Sender: TObject);
  end;

var
  fSubtractors: TfSubtractors;

implementation

{$R *.lfm}

{***************}
{ TfSubtractors }
{***************}

{ Application start: Initialisation }

procedure TfSubtractors.FormCreate(Sender: TObject);

var
  S: string;

begin
  S := 'S' + SUB_Digits[3] + ' S' + SUB_Digits[2] + ' S' + SUB_Digits[1] + ' S' + SUB_Digits[0];
  Memo3.Text := StringReplace(Memo3.Text, 'S3 S2 S1 S0', S, []);
  Memo4.Text := StringReplace(Memo4.Text, 'S3 S2 S1 S0', S, []);
end;

{ Menu item "File > Exit": Exit application }

procedure TfSubtractors.mFileExitClick(Sender: TObject);

begin
  Close;
end;

{ Menu item "Help > Help": Display maths/electronics help }

procedure TfSubtractors.mHelpHelpClick(Sender: TObject);

begin
  fHelp.Show;
end;

{ Menu item "Help > About": Display application about }

procedure TfSubtractors.mHelpAboutClick(Sender: TObject);

var
  S: string;

begin
  S := 'Electronics simulation.' + LineEnding;
  S += 'Using logic gates to make binary subtractor circuits.' + LineEnding + LineEnding;
  S += 'Version 1.0.1, © allu, February 2020 - September 2021.';
  MessageDlg('About "LogicGates4"', S, mtInformation, [mbOK], 0);
end;

{ Button "1-bit-half-subtractor": Run 1-bit-half-subtractor simulation }

procedure TfSubtractors.btSubtractor1hClick(Sender: TObject);

begin
  fSubtractor1h.ShowModal;
end;

{ Button "1-bit-full-subtractor": Run 1-bit-full-subtractor simulation }

procedure TfSubtractors.btSubtractor1fClick(Sender: TObject);

begin
  fSubtractor1f.ShowModal;
end;

{ Button "4-bit-full-subtractor": Run 4-bit-full-subtractor simulation }

procedure TfSubtractors.btSubtractor4fClick(Sender: TObject);

begin
  fSubtractor4f.ShowModal;
end;

{ Button "4-bit-full-adder-subtractor": Run 4-bit-full-adder-subtractor simulation }

procedure TfSubtractors.btAddSub4fClick(Sender: TObject);

begin
  fAddSub4f.ShowModal;
end;

end.

